function [F_e,Fval_e,Ftest_e] = secondOrderEstimation(Tdata,Tval,Ttest,N,Xdata,Fdata,Xvaldata,Fvaldata,Xtestdata,Ftestdata)
% FIRSTORDERESTIMATION  Calculates second-order parameters W to estimate
% the function F from signals in Xdata using least-squares estimation.
% There are N signals and T time steps.
%
%
% For every time step we have
%
% F_e(X) = X'*W2*X + W1'*X + w0
%
% where
% X = (x1,x2,...,xN), W1 = (w1,w2,...,wN), w0 is a scalar treshold value
% and
%       |w11 ... w1N|
% W2 =  |... ... ...|
%       |wN1 ... wNN|
%
%
% We rewrite it as
%
% F_e(X) = W*X
%
% where
% X = ( 1;   x1:xN;   x1x1:xNxN;   x1x2:x(N-1)xN;   x1x3:x(N-2)xN; ... )
% W = (w0,   w1:wN,   w11:wNN,     w12:w(N-1)N,     w13:w(N-2)N    ... )
%
% W should be found such that we minimize V = 0.5*sum( (f_e(X) - f(X))^2 )
% where X and f(X) are known vectors.
%
% F_e, Fval_e and Ftest_e are then found using W.
%
% Finally, the function returns F_e, which is the estimate of F, 
% Fval_e, which is the estimate of Fval, and Ftest_e, which is
% the estimate of Ftest.
%
% See also NEURALNETWORK, MAKEDATA, FIRSTORDERESTIMATION

% Anders Fougner, anderfo@stud.ntnu.no
% $Revision: 2.0 $Date: 2007/05/20 22:13:00 $

%% Estimation 2nd order
% Performing least-squares estimation for each of N signals

% Adjust X
X = [ones(1,Tdata)]; % add 'ones' for calculating W_0 parameters
X = [X; Xdata];
% The for-loop goes on like this:
% X = [X;(Xdata(1:N,1:T).*Xdata(1:N,1:T))];
% X = [X;(Xdata(1:N-1,1:T).*Xdata(2:N,1:T))];
% X = [X;(Xdata(1:N-2,1:T).*Xdata(3:N,1:T))];
% ...
% X = [X;(Xdata(1:1,1:T).*Xdata(N:N,1:T))];
for k=N:-1:1
    X = [X;(Xdata(1:k,1:Tdata).*Xdata(N-k+1:N,1:Tdata))];
end

%Calculate the optimal W
W = X'\Fdata';

% Calculate F_e(X), the estimate of F(x)
F_e = W'*X;

%% Simulate using validation data
% Adjust Xval in the same way as X was adjusted
Xval = [ones(1,Tval)]; % add 'ones' for calculating W_0 parameters
Xval = [Xval; Xvaldata];
for k=N:-1:1
    Xval = [Xval;(Xvaldata(1:k,1:Tval).*Xvaldata(N-k+1:N,1:Tval))];
end

% Calculate Fval_e(X), the estimate of Fval(X)
Fval_e = W'*Xval;

%% Simulate using test data
% Adjust Xtest in the same way as X was adjusted
Xtest = [ones(1,Ttest)]; % add 'ones' for calculating W_0 parameters
Xtest = [Xtest; Xtestdata];
for k=N:-1:1
    Xtest = [Xtest;(Xtestdata(1:k,1:Ttest).*Xtestdata(N-k+1:N,1:Ttest))];
end

% Calculate Ftest_e(X), the estimate of Ftest(X)
Ftest_e = W'*Xtest;